#!/usr/bin/python

import sys
import csv
import re
from collections import defaultdict
import codecs
sys.stdout = codecs.getwriter('utf8')(sys.stdout)

p = re.compile('--')
# path marking $\uparrow$ is done via '|' in lifting code
# reserved in Moses / MbotMoses for factors -> substitute with '!'
p2 = re.compile('\|')

# escape harmful symbols for Moses / MbotMoses
factor = re.compile('\|')
lt = re.compile('<')
gt = re.compile('>')
quot = re.compile('"')
apos = re.compile("'")
lnt = re.compile('\[')
rnt = re.compile('\]')
amp = re.compile('&')

def make_tree(s_id, root, words, heads, done):
    if root in done: return ""
    children = []
    for i in sorted(heads[root]):
        if i == root:
             children.append('<tree label="%s"> %s </tree>' % (words[root][2], words[root][4]))
        else:
            children.append(make_tree(s_id, i, words, heads, done + [root])) 
    return '<tree label="%s"> %s </tree>' % (words[root][1], " ".join(children))


# CoNLL09-format:
# ['0_ID', '1_Form', '2_Lemma', '3_PLemma', '4_POS', '5_PPOS', '6_Feats', '7_PFeats', '8_Head', '9_PHead', '10_Deprel', '11_PDeprel', '12_IsPred', '13_Pred']

input = sys.argv[1]
o = sys.argv[2]
outfile = open(o, 'w')

words = {}
heads = defaultdict(list)
s_id = 1
failed = 0

with open(input, 'rb') as csvfile:
    reader = csv.reader(csvfile, delimiter='\t', quoting=csv.QUOTE_NONE)
    for columns in reader:
        size = len(columns)
       
        if size == 0:
            try:
                output = ("%s" % (make_tree(s_id, 0, words, heads, []))).decode("utf8")
                outfile.write(output.encode('utf8'))
                outfile.write('\n')
                outfile.flush()
            except KeyError:
                failed += 1
                outfile.write('\n')
                
            words = {}
            heads = defaultdict(list)
            s_id += 1
        else:
            w_id = int(columns[0])
            w_head = int(columns[9])
            heads[w_id].append(w_id)
            heads[w_head].append(w_id)
            w_pos = columns[5]
            w_feat = columns[6]
            w_lemma = columns[1]

            if w_head == 0 and w_pos != "-":
                w_role = "ROOT"
                words[0] = (0, "TOP", "root", "root", "root")
            else:
                w_role = columns[11]
                
                if w_pos == "-":
                    if w_lemma == "(":
                        w_pos = "LRB"
                        w_role = "PAR"
                    elif w_lemma == ")":
                        w_pos = "RRB"
                        w_role = "PAR"
                    elif w_lemma == "/" or w_lemma == '\\' or w_lemma == '"' or w_lemma == '-':
                        w_pos = "INTJ"
                        w_role = "PUNC"
                    elif  w_lemma == "," or w_lemma == ";" or w_lemma == ":":
                        w_pos = ","
                        w_role = "PUNC"
                    else:
                        w_pos = "unk"
                        w_role = "UNK"

                uplift = p2.search(w_role)
                if uplift:
                    w_role = p2.sub('!', w_role)

            # escape special chars
            w_lemma = amp.sub('&amp;', w_lemma)
            w_lemma = factor.sub('&bar;', w_lemma)
            w_lemma = lt.sub('&lt;', w_lemma)
            w_lemma = gt.sub('&gt;', w_lemma)
            w_lemma = quot.sub('&quot;', w_lemma)
            w_lemma = apos.sub('&apos;', w_lemma)
            w_lemma = lnt.sub('&#91;', w_lemma)
            w_lemma = rnt.sub('&#93;', w_lemma)

            words[w_id] = (w_head, w_role, w_pos, w_feat, w_lemma)

print "Skipped", failed, "sentences in total!\nPlease run clean-corpus-n.perl again!!!\n" 

